package netprog.http;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Properties;
import java.util.Vector;

class MTHttpServer implements HttpConstants {
	
	static PrintStream log = null;  // log information
	
	static Vector threads = new Vector(); // Where the worker threads stand idle
	
	static File wwwRoot;
	
	static int timeout = 5000; // timeout of client connections
	
	static int maxWorkers  =  25; // maximum number of worker threads
	
	static int port = 8080; // default port
	
	protected static Properties props = new Properties(); // Server properties
	
	protected static void p(String s) {  // print to stdout
		System.out.println(s);
	}
	
	protected static void log(String s) {  // logs in logfile
		synchronized(log) {
			log.println(s);
			log.flush();
		}
	}
	
	static void loadProps() throws IOException {
		if (wwwRoot == null) {
			wwwRoot = new File(System.getProperty("user.dir"));			
		}
		if  (maxWorkers > 25) {
			maxWorkers = 5;
		}
		if (log == null) {
			p("logging to stdout");
			log = System.out;
		}
	}
	static void printProps() throws IOException {		
		p("root="+wwwRoot);
		p("timeout="+timeout);
		p("workers=" + maxWorkers);
		
	}
	
	public static void main(String[] a) throws Exception {
		int port = 8080;
		if (a.length > 0) {
			port = Integer.parseInt(a[0]);
			File f = new File(a[1]);
			wwwRoot = f;
			maxWorkers = Integer.parseInt(a[2]);			
		}
		loadProps(); //Load the web server properties
		printProps();  // print the web server properties
		/*start worker threads */
		for (int i = 0; i < maxWorkers; i++) {
			MTHttpServerWorker w = new MTHttpServerWorker();
			(new Thread(w, "worker #" + i )).start();
			threads.addElement(w);
		}
		
		ServerSocket serverSocket = new ServerSocket(port);
		while (true) {
			System.out.println("Waiting for connections");
			Socket s = serverSocket.accept();
			System.out.println("Server bound to port: " + port);
			System.out.println("Incoming connection from " + serverSocket.getInetAddress());
			
			MTHttpServerWorker w = null;
			synchronized (threads) {
				if (threads.isEmpty()) {
					MTHttpServerWorker ws = new MTHttpServerWorker();
					ws.setSocket(s);
					(new Thread(ws, "additional worker")).start();
				} else {					
					w = (MTHttpServerWorker) threads.elementAt(0);
					threads.removeElementAt(0);
					w.setSocket(s);
				}
			}
		}
	}
}