package netprog.http;

import java.io.*;
import java.net.*;
import java.util.*;


public class MTHttpServerWorker extends MTHttpServer implements HttpConstants, Runnable  {
			
	public static final int MIMETYPE_HTML = 0;
	public static final int MIMETYPE_JPEG = 2;
	public static final int MIMETYPE_ZIP = 3;
	public static final int MIMETYPE_GIF = 4;
	public static final int MIMETYPE_PNG = 5;
	public static final int MIMETYPE_MP3 = 6;
	public static final int MIMETYPE_ASF = 7;
	
	private Socket s;

	
	synchronized void setSocket(Socket s) {
		this.s = s;
		notify();
	}
	
	public synchronized void run() {
		
		while(true) {
			if(s==null) {
				/*nothing to do*/
				try {
					wait();					
				} catch (InterruptedException e) {
					/* should not happen */
					continue;
				}
			}
			try {			
				protocolHandler(); 
			} catch (Exception e) {
				e.printStackTrace();
			}
			/* go back in wait queue if there is fewer than numHandler connections */
			s = null;
			Vector pool = MTHttpServer.threads;
			synchronized(pool) {
				if (pool.size() >= MTHttpServer.maxWorkers) {
					/*too many threads, exit that one */
					return;
				} else {
					pool.addElement(this);
				}
			}
		}
	}
	
	private void protocolHandler() {
		try {
			ServerSocket serverSocket = new ServerSocket(port);	
			Socket clientConnection = serverSocket.accept();
			BufferedReader bis = new BufferedReader(new InputStreamReader(clientConnection.getInputStream()));
			DataOutputStream dos = new DataOutputStream(clientConnection.getOutputStream());
			//InputStream is = new BufferedInputStream(s.getInputStream());
			//PrintStream ps = new PrintStream(s.getOutputStream());
			String request = bis.readLine();
			System.out.println(request);
			//processing the request

			if (request == null) {
				//null request is not valid
				dos.writeBytes(constructHTTPHeader(403, MIMETYPE_HTML));
				dos.close();
				System.out.println("Request is null");
				return;
			}

			//process the request
			StringTokenizer strTok = new StringTokenizer(request, " ");
			String reqMethod = strTok.nextToken();
			String resourceName = strTok.nextToken().substring(1); //remove slash

			if (reqMethod.equals("GET")) {
				try {
					System.out.println("Client requested resource:" + new File(wwwRoot + resourceName).getAbsolutePath());
					FileInputStream fis = new FileInputStream(wwwRoot + resourceName);
					//check the type of resource
					if (supportedResource(resourceName)) {
						dos.writeBytes(constructHTTPHeader(200, getMimeType(wwwRoot + resourceName)));
						while (true) { //write the resource to the stream
							int b = fis.read();
							if (b == -1) {
								break; // end of file
							}
							dos.write(b);
						}
						dos.close();
						fis.close();
						return;
					} else { //media type not supported
						dos.writeBytes(constructHTTPHeader(403, MIMETYPE_HTML)); //access to this resource is forbidden
						dos.close();
						System.out.println("Resource type not supported:" + resourceName);
						return;
					}


				} catch(FileNotFoundException fnf) {
					dos.writeBytes(constructHTTPHeader(404, MIMETYPE_HTML));
					dos.close();
					System.out.println("Resource not found");
					return;
				} catch(SecurityException se) {
					dos.writeBytes(constructHTTPHeader(403, MIMETYPE_HTML));
					System.out.println("Access is denied");
					dos.close();
					return;
				}catch(Exception e) {
					dos.writeBytes(constructHTTPHeader(500, MIMETYPE_HTML));
					dos.close();
					return;
				}
			} else {
				dos.writeBytes(constructHTTPHeader(501, 0)); //not implemented
				dos.close();
				return;
			}



		} catch(Exception e) {
			System.out.println("ERROR" + e.toString());
		}
	}
	
	private boolean supportedResource(String resource) {
		String cResource = resource.toUpperCase();

		if(cResource.endsWith(".JPEG") || 
				cResource.endsWith(".JPG") ||
				cResource.endsWith(".PNG") ||
				cResource.endsWith(".GIF") ||
				cResource.endsWith(".ZIP") ||
				cResource.endsWith("MP3")  ||
				cResource.endsWith("WMV")  ||
				cResource.endsWith(".HTML")) {
			return true;
		} else {
			return false;
		}
	}

	private int getMimeType(String resource) {
		String cResource = resource.toUpperCase();

		if (cResource.endsWith(".JPG") || cResource.endsWith(".JPEG")) 
			return this.MIMETYPE_JPEG;
		else if (cResource.endsWith(".GIF") ) 
			return this.MIMETYPE_GIF;
		else if (cResource.endsWith(".PNG")) 
			return this.MIMETYPE_PNG;
		else if (cResource.endsWith(".ZIP")) 
			return this.MIMETYPE_ZIP;
		else if (cResource.endsWith(".MP3")) 
			return this.MIMETYPE_MP3;
		else if (cResource.endsWith(".WMV")) 
			return this.MIMETYPE_ASF;
		else //html
			return this.MIMETYPE_HTML;
	}
	
	private String constructHTTPHeader(int replyCode, int mimeType) {
		String replyHeader = "HTTP/1.1 ";

		switch(replyCode) {
		case 200:
			replyHeader += "200 OK";
			break;
		case 400:
			replyHeader += "400 Bad Request";
			break;
		case 404:
			replyHeader += "404 Not Found";
			break;
		case 403:
			replyHeader += "403 Forbidden";
			break;
		case 501:
			replyHeader += "501 NOT Implemented";
			break;
		case 500:
			replyHeader += "500 Internal Server error";
			break;
		}
		replyHeader += "\r\n";
		Date d = new Date();
		replyHeader += "Date:" + d.toString() + "\r\n";
		//replyHeader += "Server:" + serverVersion + "\r\n";



		switch(mimeType) {
		case MIMETYPE_JPEG:
			replyHeader  += "Content-Type: image/jpeg\r\n";
			break;
		case MIMETYPE_PNG:
			replyHeader  += "Content-Type: image/png\r\n";
			break;
		case MIMETYPE_GIF:
			replyHeader  += "Content-Type: image/GIF\r\n";
			break;
		case MIMETYPE_ZIP:
			replyHeader += "Content-Type: application/zip\r\n";
			break;
		case MIMETYPE_MP3:
			replyHeader  += "Content-Type: audio/mp3\r\n";
			break;
		case MIMETYPE_ASF:
			replyHeader += "Content-Type: video/x-ms-asf\r\n";
			break;
		default:
			replyHeader  += "Content-Type: text/html\r\n";
			break;
		}
		
		replyHeader += "\r\n";
		System.out.println(replyHeader);
		return replyHeader;
	}	
}